@@ -1,7 +1,7 @@
 /*
  * GNetwork Library: libgnetwork/gnetwork-ssl.c
  *
- * Copyright (C) 2003 James M. Cape
+ * Copyright (C) 2003, 2004 James M. Cape
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License as
@@ -45,6 +45,7 @@
 
 #if defined _USE_GNUTLS
 # include <gnutls/gnutls.h>
+# include <gnutls/x509.h>
 # define GNUTLS_DATUM(ptr)	((gnutls_datum *) (ptr))
 #elif defined _USE_OPENSSL
 # include <openssl/ssl.h>
@@ -65,19 +66,18 @@
 
 struct _GNetworkSslCert
 {
+  GTypeClass g_class;
+
   gchar *name;
   gchar *email;
-
   gchar *organization;
   gchar *department;
-
   gchar *city;
   gchar *province;
   gchar *country;
 
-  glong expiration;
-  glong activation;
-  glong issued;
+  gint64 expiration;
+  gint64 activation;
 
   GNetworkSslCertType cert_type:2;
 };
@@ -104,9 +104,6 @@ get_ssl_cert_error_from_error_num (gint error_num)
       if (error_num & GNUTLS_CERT_INVALID)
 	error |= GNETWORK_SSL_CERT_ERROR_INVALID;
 
-      if (error_num & GNUTLS_CERT_CORRUPTED)
-	error |= GNETWORK_SSL_CERT_ERROR_CORRUPTED;
-
       if (error_num & GNUTLS_CERT_REVOKED)
 	error |= GNETWORK_SSL_CERT_ERROR_REVOKED;
     }
@@ -122,48 +119,73 @@ get_ssl_cert_error_from_error_num (gint error_num)
 }
 
 
-#if defined _USE_GNUTLS
-/* GNUTLS Version */
+# ifdef _USE_GNUTLS
+
+static gchar *
+get_oid (const gnutls_x509_crt x509cert, const gchar *oid)
+{
+  gchar *retval;
+  gsize buffer_size;
+
+  if (gnutls_x509_crt_get_dn_by_oid (x509cert, oid, 0, 0, NULL, &buffer_size)
+      != GNUTLS_E_SHORT_MEMORY_BUFFER)
+    {
+      retval = NULL;
+    }
+  else
+    {
+      retval = g_new (gchar, buffer_size);
+
+      if (gnutls_x509_crt_get_dn_by_oid (x509cert, GNUTLS_OID_X520_COMMON_NAME, 0, 0, retval,
+					 &buffer_size) != 0)
+	{
+	  g_free (retval);
+	  retval = NULL;
+	}
+    }
+
+  return retval;
+}
 
 static GNetworkSslCert *
-_gnetwork_ssl_cert_new (const gnutls_datum * data)
+_gnetwork_ssl_cert_new (const gnutls_x509_crt x509cert, GNetworkSslCertType type)
 {
   GNetworkSslCert *cert;
-  gnutls_x509_dn dn;
 
-  cert = g_new0 (GNetworkSslCert, 1);
-  cert->cert_type = GNETWORK_SSL_CERT_X509;
+  cert = g_new (GNetworkSslCert, 1);
+  cert->g_class.g_type = GNETWORK_TYPE_SSL_CERT;
 
-  gnutls_x509_extract_certificate_dn (data, &dn);
+  cert->cert_type = type;
 
-  cert->name = g_strdup (dn.common_name);
-  cert->email = g_strdup (dn.email);
-
-  cert->organization = g_strdup (dn.organization);
-  cert->department = g_strdup (dn.organizational_unit_name);
-
-  cert->city = g_strdup (dn.locality_name);
-  cert->province = g_strdup (dn.state_or_province_name);
-  cert->country = g_strdup (dn.country);
-
-  cert->activation = gnutls_x509_extract_certificate_activation_time (data);
-  cert->expiration = gnutls_x509_extract_certificate_expiration_time (data);
+  cert->name = get_oid (x509cert, GNUTLS_OID_X520_COMMON_NAME);
+  cert->email = get_oid (x509cert, GNUTLS_OID_PKCS9_EMAIL);
+  cert->organization = get_oid (x509cert, GNUTLS_OID_X520_ORGANIZATION_NAME);
+  cert->department = get_oid (x509cert, GNUTLS_OID_X520_ORGANIZATIONAL_UNIT_NAME);
+  cert->city = get_oid (x509cert, GNUTLS_OID_X520_LOCALITY_NAME);
+  cert->province = get_oid (x509cert, GNUTLS_OID_X520_STATE_OR_PROVINCE_NAME);
+  cert->country = get_oid (x509cert, GNUTLS_OID_X520_COUNTRY_NAME);
+  cert->expiration = gnutls_x509_crt_get_expiration_time (x509cert);
+  cert->activation = gnutls_x509_crt_get_activation_time (x509cert);
 
   return cert;
 }
 
-#elif defined _USE_OPENSSL
-/* OpenSSL Version */
+# elif defined _USE_OPENSSL
 
 static GNetworkSslCert *
-_gnetwork_ssl_cert_new (const X509 * data)
+_gnetwork_ssl_cert_new (X509 *x509cert, GNetworkSslCertType type)
 {
-  GNetworkSslCert *cert;
 
   return cert;
 }
 
-#endif /* _USE_GNUTLS, _USE_OPENSSL */
+# else
+
+
+
+# endif /* _USE_OPENSSL */
+
+
 
 #endif /* _USE_SSL */
 /* End SSL-Only */
@@ -201,29 +223,26 @@ gnetwork_ssl_cert_get_type (void)
  * Since: 1.0
  **/
 GNetworkSslCert *
-gnetwork_ssl_cert_dup (GNetworkSslCert * cert)
+gnetwork_ssl_cert_dup (const GNetworkSslCert * cert)
 {
 #ifdef _USE_SSL
   GNetworkSslCert *retval;
 
-  if (cert == NULL)
-    return NULL;
+  g_return_val_if_fail (cert == NULL || GNETWORK_IS_SSL_CERT (cert), NULL);
 
-  retval = g_new0 (GNetworkSslCert, 1);
-
-  retval->cert_type = cert->cert_type;
-  retval->name = g_strdup (cert->name);
-  retval->email = g_strdup (cert->email);
-
-  retval->organization = g_strdup (cert->organization);
-  retval->department = g_strdup (cert->department);
-  retval->city = g_strdup (cert->city);
-  retval->province = g_strdup (cert->province);
-  retval->country = g_strdup (cert->country);
-
-  retval->expiration = cert->expiration;
-  retval->activation = cert->activation;
-  retval->issued = cert->issued;
+  if (cert != NULL)
+    {
+      retval = g_memdup (cert, sizeof (GNetworkSslCert));
+      retval->name = g_strdup (cert->name);
+      retval->email = g_strdup (cert->email);
+      retval->organization = g_strdup (cert->organization);
+      retval->department = g_strdup (cert->department);
+      retval->city = g_strdup (cert->city);
+      retval->province = g_strdup (cert->province);
+      retval->country = g_strdup (cert->country);
+    }
+  else
+    retval = NULL;
 
   return retval;
 #else
@@ -244,12 +263,13 @@ void
 gnetwork_ssl_cert_free (GNetworkSslCert * cert)
 {
 #ifdef _USE_SSL
+  g_return_if_fail (cert == NULL || GNETWORK_IS_SSL_CERT (cert));
+
   if (cert == NULL)
     return;
 
   g_free (cert->name);
   g_free (cert->email);
-
   g_free (cert->organization);
   g_free (cert->department);
   g_free (cert->city);
@@ -272,7 +292,7 @@ gnetwork_ssl_cert_free (GNetworkSslCert * cert)
  * Since: 1.0
  **/
 GNetworkSslCertType
-gnetwork_ssl_cert_get_cert_type (GNetworkSslCert * cert)
+gnetwork_ssl_cert_get_cert_type (const GNetworkSslCert * cert)
 {
 #ifdef _USE_SSL
   g_return_val_if_fail (cert != NULL, GNETWORK_SSL_CERT_INVALID);
@@ -295,10 +315,11 @@ gnetwork_ssl_cert_get_cert_type (GNetworkSslCert * cert)
  * Since: 1.0
  **/
 G_CONST_RETURN gchar *
-gnetwork_ssl_cert_get_name (GNetworkSslCert * cert)
+gnetwork_ssl_cert_get_name (const GNetworkSslCert * cert)
 {
 #ifdef _USE_SSL
-  g_return_val_if_fail (cert != NULL, NULL);
+
+  g_return_val_if_fail (GNETWORK_IS_SSL_CERT (cert), NULL);
   g_return_val_if_fail (cert->cert_type == GNETWORK_SSL_CERT_X509, NULL);
 
   return cert->name;
@@ -307,7 +328,7 @@ gnetwork_ssl_cert_get_name (GNetworkSslCert * cert)
 #endif /* _USE_SSL */
 }
 
-
+#if 0
 /**
  * gnetwork_ssl_cert_get_email:
  * @cert: the certificate to examine.
@@ -457,6 +478,8 @@ gnetwork_ssl_cert_get_country (GNetworkSslCert * cert)
 #endif /* _USE_SSL */
 }
 
+#endif /* 0 */
+
 
 /**
  * gnetwork_ssl_cert_get_expiration:
@@ -469,7 +492,7 @@ gnetwork_ssl_cert_get_country (GNetworkSslCert * cert)
  * Since: 1.0
  **/
 glong
-gnetwork_ssl_cert_get_expiration (GNetworkSslCert * cert)
+gnetwork_ssl_cert_get_expiration (const GNetworkSslCert * cert)
 {
 #ifdef _USE_SSL
   g_return_val_if_fail (cert != NULL, -1);
@@ -494,31 +517,7 @@ gnetwork_ssl_cert_get_expiration (GNetworkSslCert * cert)
  * Since: 1.0
  **/
 glong
-gnetwork_ssl_cert_get_activation (GNetworkSslCert * cert)
-{
-#ifdef _USE_SSL
-  g_return_val_if_fail (cert != NULL, -1);
-  g_return_val_if_fail (cert->cert_type == GNETWORK_SSL_CERT_X509, -1);
-
-  return cert->activation;
-#else
-  return -1;
-#endif /* _USE_SSL */
-}
-
-
-/**
- * gnetwork_ssl_cert_get_issued:
- * @cert: the certificate to examine.
- * 
- * Retrieves the UNIX time_t (seconds since the epoch) when @cert was created.
- * 
- * Returns: the creation time of @cert.
- * 
- * Since: 1.0
- **/
-glong
-gnetwork_ssl_cert_get_issued (GNetworkSslCert * cert)
+gnetwork_ssl_cert_get_activation (const GNetworkSslCert * cert)
 {
 #ifdef _USE_SSL
   g_return_val_if_fail (cert != NULL, -1);
@@ -857,14 +856,8 @@ _gnetwork_ssl_auth_new (GNetworkSslAuthType auth_type, GNetworkConnectionType co
 
 	  if (dh_params == NULL)
 	    {
-	      gnutls_datum prime, generator;
-
 	      gnutls_dh_params_init ((gnutls_dh_params *) &dh_params);
-	      gnutls_dh_params_generate (&prime, &generator, 1024);
-	      gnutls_dh_params_set (dh_params, prime, generator, 1024);
-
-	      free (prime.data);
-	      free (generator.data);
+	      gnutls_dh_params_generate2 (dh_params, 1024);
 	    }
 
 	  gnutls_certificate_set_dh_params (auth->cred.certificate, dh_params);
@@ -1357,26 +1350,28 @@ perform_handshake (GNetworkIOChannelSsl * channel, GError ** error)
 		   i < cert_list_size && status != G_IO_STATUS_NORMAL; i++)
 		{
 		  glong current_time;
+		  gnutls_x509_crt x509cert;
 		  GNetworkSslCert *cert;
 
+		  gnutls_x509_crt_init (&x509cert);
+		  gnutls_x509_crt_import (x509cert, certs + i, GNUTLS_X509_FMT_DER);
 		  current_time = time (NULL);
-		  cert = _gnetwork_ssl_cert_new (certs + i);
+		  cert = _gnetwork_ssl_cert_new (x509cert, GNETWORK_SSL_CERT_X509);
 
-		  if (cert->expiration < current_time)
+		  if (gnetwork_ssl_cert_get_expiration (cert) < current_time)
 		    {
 		      errors |= GNETWORK_SSL_CERT_ERROR_EXPIRED;
 		      status = G_IO_STATUS_ERROR;
 		    }
 
-		  if (cert->activation > current_time)
+		  if (gnetwork_ssl_cert_get_activation (cert) > current_time)
 		    {
 		      errors |= GNETWORK_SSL_CERT_ERROR_NOT_ACTIVATED;
 		      status = G_IO_STATUS_ERROR;
 		    }
 
 		  if (channel->auth->server_hostname != NULL &&
-		      !gnutls_x509_check_certificates_hostname (certs + i,
-								channel->auth->server_hostname))
+		      !gnutls_x509_crt_check_hostname (x509cert, channel->auth->server_hostname))
 		    {
 		      errors |= GNETWORK_SSL_CERT_ERROR_HOSTNAME_MISMATCH;
 		      status = G_IO_STATUS_ERROR;
@@ -1692,7 +1687,6 @@ _gnetwork_io_channel_ssl_new (GIOChannel * parent, GNetworkSslAuth * auth, GErro
     GNUTLS_CIPHER_RIJNDAEL_128_CBC,
     GNUTLS_CIPHER_3DES_CBC,
     GNUTLS_CIPHER_RIJNDAEL_256_CBC,
-    GNUTLS_CIPHER_TWOFISH_128_CBC,
     GNUTLS_CIPHER_ARCFOUR,
     0
   };
